import { EventEmitter, Injectable, Input, Output, Component, ViewEncapsulation, ChangeDetectorRef, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ChartItemService {
    constructor() {
        this.values = [];
        this.chartValueChange = new EventEmitter();
        this.chartValueHover = new EventEmitter();
        this.chartValueLeave = new EventEmitter();
        this.chartValueSelect = new EventEmitter();
        this.chartValueDeselect = new EventEmitter();
    }
    /**
     * @param {?} val
     * @return {?}
     */
    setChartValues(val) {
        /** @type {?} */
        var values = this.values.find((/**
         * @param {?} item
         * @return {?}
         */
        item => {
            if (item.chartId === val.chartId) {
                return true;
            }
            return false;
        }));
        if (values) {
            values = val;
        }
        else {
            this.values.push(val);
        }
        this.chartValueChange.emit(val);
    }
    /**
     * @param {?} chartId
     * @return {?}
     */
    getChartValues(chartId) {
        return this.values.find((/**
         * @param {?} item
         * @return {?}
         */
        item => {
            if (item.chartId === chartId) {
                return true;
            }
            return false;
        }));
    }
    /**
     * @param {?} val
     * @return {?}
     */
    hoverChartValue(val) {
        this.chartValueHover.emit(val);
    }
    /**
     * @param {?} val
     * @return {?}
     */
    leaveChartValue(val) {
        this.chartValueLeave.emit(val);
    }
    /**
     * @param {?} val
     * @return {?}
     */
    selectChartValue(val) {
        this.chartValueSelect.emit(val);
    }
    /**
     * @param {?} val
     * @return {?}
     */
    deselectChartValue(val) {
        this.chartValueDeselect.emit(val);
    }
}
ChartItemService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ChartItemService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TooltipConfiguration {
    /**
     * @param {?=} HideValue
     * @param {?=} ValueFunction
     */
    constructor(HideValue = false, ValueFunction = null) {
        this.HideValue = false;
        this.ValueFunction = null;
        this.HideValue = HideValue;
        this.ValueFunction = ValueFunction;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class ServiceItem {
    /**
     * @param {?} chartId
     * @param {?} value
     * @param {?} tooltipX
     * @param {?} tooltipY
     */
    constructor(chartId, value, tooltipX, tooltipY) {
        this.chartId = chartId;
        this.value = value;
        this.tooltipX = tooltipX;
        this.tooltipY = tooltipY;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class BaseChartComponent {
    /**
     * @param {?} chartItemService
     * @param {?} utilsService
     * @param {?} cd
     */
    constructor(chartItemService, utilsService, cd) {
        this.chartItemService = chartItemService;
        this.utilsService = utilsService;
        this.cd = cd;
        this.currentValues = [];
        this.tooltipShow = false;
        this.tooltipClass = '';
        this.animationActive = true;
        this.valueSelect = new EventEmitter();
        this.valueDeselect = new EventEmitter();
        this.valueClick = new EventEmitter();
        this.valueChange = new EventEmitter();
        this.chartidChange = new EventEmitter();
        this.currentChartid = 'dl-chart-1';
        this.currentTooltipConfiguration = null;
        this.currentHideTooltip = false;
        this.currentAllowSelect = true;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set values(val) {
        this.currentValues = val;
        this.cd.detectChanges();
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set animation(val) {
        this.animationActive = val;
    }
    /**
     * @return {?}
     */
    get chartid() {
        return this.currentChartid;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set chartid(val) {
        this.currentChartid = val;
        this.chartidChange.emit(this.currentChartid);
        this.cd.detectChanges();
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set tooltipConfiguration(val) {
        this.currentTooltipConfiguration = val;
        this.cd.detectChanges();
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set hideTooltip(val) {
        this.currentHideTooltip = val;
        this.cd.detectChanges();
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set allowSelect(val) {
        this.currentAllowSelect = val;
        this.cd.detectChanges();
    }
    /**
     * @return {?}
     */
    get tooltipValue() {
        if (this.tooltipContentItem) {
            if (this.tooltipContentItem.tooltipConfig != null) {
                return this.utilsService.textValue(this.tooltipContentItem.tooltipConfig, this.tooltipContentItem, this.tooltipContentChartItem.calculatedPercent);
            }
            else {
                return this.utilsService.textValue(this.currentTooltipConfiguration, this.tooltipContentItem, this.tooltipContentChartItem.calculatedPercent);
            }
        }
        return null;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onClickSegment(event) {
        if (this.currentAllowSelect) {
            if (event === this.currentActiveChartItem) {
                this.currentActiveChartItem = null;
                this.valueDeselect.emit(event.sourceItem);
                this.chartItemService.chartValueDeselect.emit(new ServiceItem(this.chartid, event, null, null));
            }
            else {
                this.currentActiveChartItem = event;
                this.valueSelect.emit(event.sourceItem);
                this.chartItemService.chartValueSelect.emit(new ServiceItem(this.chartid, event, null, null));
            }
        }
        this.valueClick.emit(event.sourceItem);
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    onHoverSegment(event, item) {
        this.tooltipContentChartItem = item;
        this.tooltipContentItem = item.sourceItem;
        this.tooltipLeft = (event.clientX + 10);
        this.tooltipTop = (event.clientY + 10);
        if (this.currentHideTooltip === false) {
            this.tooltipShow = true;
        }
        else {
            this.tooltipShow = false;
        }
        this.valueChange.emit(item.sourceItem);
        this.chartItemService.chartValueHover.emit(new ServiceItem(this.chartid, item, this.tooltipLeft, this.tooltipTop));
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onLeaveSegment(event) {
        this.tooltipContentChartItem = null;
        this.tooltipContentItem = null;
        this.tooltipShow = false;
        this.valueChange.emit(null);
        this.chartItemService.chartValueLeave.emit(new ServiceItem(this.chartid, null, null, null));
    }
    /**
     * @return {?}
     */
    resetActive() {
        this.tooltipContentChartItem = null;
        this.tooltipContentItem = null;
        this.tooltipShow = false;
        this.valueChange.emit(null);
    }
}
BaseChartComponent.propDecorators = {
    values: [{ type: Input }],
    animation: [{ type: Input }],
    valueSelect: [{ type: Output }],
    valueDeselect: [{ type: Output }],
    valueClick: [{ type: Output }],
    valueChange: [{ type: Output }],
    chartidChange: [{ type: Output }],
    chartid: [{ type: Input }],
    tooltipConfiguration: [{ type: Input }],
    hideTooltip: [{ type: Input }],
    allowSelect: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class UtilsService {
    constructor() {
    }
    /**
     * @param {?} config
     * @param {?} item
     * @param {?} percent
     * @return {?}
     */
    textValue(config, item, percent) {
        if (config != null) {
            if (config.ValueFunction != null) {
                return config.ValueFunction(item, percent);
            }
            else {
                if (config.HideValue) {
                    return item.name;
                }
                else {
                    return this.getTextValue(item);
                }
            }
        }
        return this.getTextValue(item);
    }
    /**
     * @private
     * @param {?} item
     * @return {?}
     */
    getTextValue(item) {
        /** @type {?} */
        var valueItem = (/** @type {?} */ (item));
        if (valueItem.value != undefined) {
            return item.name + ' (' + valueItem.value + ')';
        }
        /** @type {?} */
        var pointItem = (/** @type {?} */ (item));
        if (pointItem.yValue != undefined) {
            return item.name + ' (' + pointItem.yValue + ', ' + pointItem.xValue + ')';
        }
        return item.name;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    roundScale(value) {
        return Math.round(value * 100) / 100;
    }
    /**
     * @param {?} prefix
     * @param {?} index
     * @return {?}
     */
    createElementId(prefix, index) {
        /** @type {?} */
        let id = prefix + index;
        return id;
    }
    /**
     * @param {?} minValue
     * @param {?} valueSteps
     * @param {?} singleStepValue
     * @param {?} singleStep
     * @param {?} maxValue
     * @return {?}
     */
    createYAxis(minValue, valueSteps, singleStepValue, singleStep, maxValue) {
        /** @type {?} */
        let axis = [];
        axis.push({
            text: this.onFormatUnkownAxisValue(minValue),
            position: 100
        });
        /** @type {?} */
        let index = 1;
        for (index = 1; index <= (valueSteps - 1); index++) {
            /** @type {?} */
            var currentValue = this.roundScale(minValue + (singleStepValue * index));
            /** @type {?} */
            var step = 100 - this.roundScale(singleStep * index);
            axis.push({
                text: this.onFormatUnkownAxisValue(currentValue),
                position: step
            });
        }
        axis.push({
            text: this.onFormatUnkownAxisValue(maxValue),
            position: 0
        });
        return axis;
    }
    /**
     * @private
     * @param {?} value
     * @return {?}
     */
    onFormatUnkownAxisValue(value) {
        if (value != undefined && value != null && !isNaN(value) && value != Infinity && value != -Infinity) {
            return value.toString();
        }
        else {
            return "";
        }
    }
}
UtilsService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
UtilsService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PieChartComponent extends BaseChartComponent {
    /**
     * @param {?} chartItemService
     * @param {?} utilsService
     * @param {?} cd
     */
    constructor(chartItemService, utilsService, cd) {
        super(chartItemService, utilsService, cd);
        this.donutRadius = 0;
        this.slices = [];
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set donut(val) {
        this.donutRadius = val;
    }
    /**
     * @return {?}
     */
    get pie() {
        return this.slices;
    }
    /**
     * @param {?} slice
     * @param {?} index
     * @return {?}
     */
    cssClassSegment(slice, index) {
        /** @type {?} */
        let css = '';
        css += 'slice';
        if (slice === this.currentActiveChartItem) {
            css += ' slice-selected';
        }
        if (slice.sourceItem.cssClass != null) {
            css += ' ' + slice.sourceItem.cssClass;
        }
        if (this.animationActive) {
            if (index % 2 == 0) {
                css += ' ' + 'slice-anim-even';
            }
            else {
                css += ' ' + 'slice-anim';
            }
        }
        return css;
    }
    /**
     * @param {?} slice
     * @param {?} index
     * @return {?}
     */
    createElementId(slice, index) {
        /** @type {?} */
        let id = this.utilsService.createElementId('chart-slice-', index);
        slice.id = id;
        return id;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.resetActiveElement();
        this.calculateSlices();
    }
    /**
     * @return {?}
     */
    calculateSlices() {
        /** @type {?} */
        let cumulativePercent = 0;
        /** @type {?} */
        let totalValue = 0;
        this.slices = [];
        this.currentValues.forEach((/**
         * @param {?} item
         * @return {?}
         */
        item => {
            if (!item.value) {
                item.value = 0;
            }
            totalValue += item.value;
        }));
        this.currentValues.forEach((/**
         * @param {?} item
         * @return {?}
         */
        item => {
            cumulativePercent = this.createSlice(item, totalValue, cumulativePercent);
        }));
        this.chartItemService.setChartValues(new ServiceItem(this.chartid, this.pie, null, null));
    }
    /**
     * @param {?} item
     * @param {?} totalValue
     * @param {?} cumulativePercent
     * @return {?}
     */
    createSlice(item, totalValue, cumulativePercent) {
        const [startX, startY] = this.getCoordinatesForPercent(cumulativePercent);
        /** @type {?} */
        let normValue = this.getNormalizedValue(totalValue, item.value);
        if (!normValue) {
            normValue = 0;
        }
        cumulativePercent += normValue;
        const [endX, endY] = this.getCoordinatesForPercent(cumulativePercent);
        /** @type {?} */
        const largeArcFlag = this.getNormalizedValue(totalValue, item.value) > .5 ? 1 : 0;
        /** @type {?} */
        var pathData = [
            `M ${startX} ${startY}`,
            `A 1 1 0 ${largeArcFlag} 1 ${endX} ${endY}`,
            `L 0 0`,
        ].join(' ');
        if (normValue === 0) {
            pathData = null;
        }
        this.slices.push({
            color: item.color,
            draw: pathData,
            sourceItem: item,
            id: null,
            calculatedPercent: this.utilsService.roundScale((normValue * 100))
        });
        return cumulativePercent;
    }
    /**
     * @param {?} percent
     * @return {?}
     */
    getCoordinatesForPercent(percent) {
        /** @type {?} */
        const x = Math.cos(2 * Math.PI * percent);
        /** @type {?} */
        const y = Math.sin(2 * Math.PI * percent);
        return [x, y];
    }
    /**
     * @param {?} totalValue
     * @param {?} value
     * @return {?}
     */
    getNormalizedValue(totalValue, value) {
        /** @type {?} */
        let percent = value / (totalValue / 100);
        if (percent < 0) {
            return 0;
        }
        return percent / 100;
    }
    /**
     * @return {?}
     */
    resetActiveElement() {
        super.resetActive();
    }
}
PieChartComponent.decorators = [
    { type: Component, args: [{
                selector: 'dl-pie-chart',
                template: "<ng-container>\r\n  <svg viewBox=\"-1 -1 2 2\" style=\"transform: rotate(-90deg);height: 100%;\" class=\"pie-chart-svg\" id=\"piechart\">\r\n    <path *ngFor=\"let slice of pie; let i = index\" [attr.id]=\"createElementId(slice, i)\" (mousemove)=\"onHoverSegment($event, slice)\"\r\n      (mouseleave)=\"onLeaveSegment($event)\" (click)=\"onClickSegment(slice)\" [attr.class]=\"cssClassSegment(slice, i)\"\r\n      [attr.d]=\"slice.draw\" [attr.fill]=\"slice.color\"></path>\r\n    <circle *ngIf=\"donutRadius > 0\" class=\"inner-circle\" [attr.r]=\"donutRadius\" cx=\"0\" cy=\"0\" />\r\n  </svg>\r\n  <div id=\"dlChartTooltip\" *ngIf=\"tooltipShow\" [style.top.px]=\"tooltipTop\" [style.left.px]=\"tooltipLeft\" class=\"tooltiptext\">\r\n    <span>{{ tooltipValue }}</span>\r\n  </div>\r\n</ng-container>",
                encapsulation: ViewEncapsulation.Emulated,
                styles: [".tooltiptext{position:fixed;overflow:hidden;text-align:center;padding:5px 10px;border-radius:6px;z-index:80;background-color:#555;color:#fff}.pie-chart-svg{overflow:visible}.slice{-webkit-transform:scale(1,1);transform:scale(1,1);transition:.5s}.slice:hover{opacity:.8;-webkit-transform:scale(1.02,1.02);transform:scale(1.02,1.02);transition:.5s}.slice-selected{-webkit-transform:scale(1.02,1.02);transform:scale(1.02,1.02);transition:.5s}.slice-anim{animation:.4s reverse scaleSlice}.slice-anim-even{animation:.5s reverse scaleSlice}.inner-circle{fill:#fff}@-webkit-keyframes scaleSlice{100%{-webkit-transform:scale(0,0);transform:scale(0,0)}}@keyframes scaleSlice{100%{-webkit-transform:scale(0,0);transform:scale(0,0)}}"]
            }] }
];
/** @nocollapse */
PieChartComponent.ctorParameters = () => [
    { type: ChartItemService },
    { type: UtilsService },
    { type: ChangeDetectorRef }
];
PieChartComponent.propDecorators = {
    donut: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DlPieChartModule {
}
DlPieChartModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    PieChartComponent
                ],
                providers: [
                    ChartItemService,
                    UtilsService
                ],
                entryComponents: [
                    PieChartComponent
                ],
                exports: [
                    PieChartComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LegendConfiguration {
    /**
     * @param {?=} HideValue
     * @param {?=} ValueFunction
     */
    constructor(HideValue = false, ValueFunction = null) {
        this.HideValue = false;
        this.ValueFunction = null;
        this.HideValue = HideValue;
        this.ValueFunction = ValueFunction;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LegendComponent {
    /**
     * @param {?} chartItemService
     * @param {?} utilsService
     * @param {?} cd
     */
    constructor(chartItemService, utilsService, cd) {
        this.chartItemService = chartItemService;
        this.utilsService = utilsService;
        this.cd = cd;
        this.legendClick = new EventEmitter();
        this.items = null;
        this.hoverItem = null;
        this.selectedItem = null;
        this.shouldHideChartHoverEffect = false;
        this.shouldHideChartSelectEffect = false;
        this.chartidChange = new EventEmitter();
        this.currentChartid = 'dl-chart-1';
        this.currentLegendConfiguration = null;
    }
    /**
     * @return {?}
     */
    get chartid() {
        return this.currentChartid;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set chartid(val) {
        this.currentChartid = val;
        this.chartidChange.emit(this.currentChartid);
    }
    /**
     * @return {?}
     */
    get chartItems() {
        if (this.items) {
            return this.items.value;
        }
        return [];
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set hideChartHoverEffect(val) {
        this.shouldHideChartHoverEffect = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set hideChartSelectEffect(val) {
        this.shouldHideChartSelectEffect = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set legendConfiguration(val) {
        this.currentLegendConfiguration = val;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.items = this.chartItemService.getChartValues(this.chartid);
        this.chartItemService.chartValueChange.subscribe((/**
         * @param {?} items
         * @return {?}
         */
        items => {
            if (items.chartId === this.chartid) {
                this.items = items;
            }
        }));
        this.chartItemService.chartValueSelect.subscribe((/**
         * @param {?} item
         * @return {?}
         */
        (item) => {
            if (item.chartId === this.chartid) {
                this.selectedItem = item.value;
            }
        }));
        this.chartItemService.chartValueDeselect.subscribe((/**
         * @param {?} item
         * @return {?}
         */
        (item) => {
            if (item.chartId === this.chartid) {
                this.selectedItem = null;
            }
        }));
        this.chartItemService.chartValueHover.subscribe((/**
         * @param {?} item
         * @return {?}
         */
        (item) => {
            if (item.chartId === this.chartid) {
                this.hoverItem = item.value;
            }
        }));
        this.chartItemService.chartValueLeave.subscribe((/**
         * @param {?} item
         * @return {?}
         */
        (item) => {
            if (item.chartId === this.chartid) {
                this.hoverItem = null;
            }
        }));
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
    }
    /**
     * @return {?}
     */
    ngAfterContentChecked() {
        this.cd.detectChanges();
    }
    /**
     * @param {?} item
     * @return {?}
     */
    legendTextValue(item) {
        return this.utilsService.textValue(this.currentLegendConfiguration, item.sourceItem, item.calculatedPercent);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    tooltipTextValue(item) {
        return this.utilsService.textValue(item.sourceItem.tooltipConfig, item.sourceItem, item.calculatedPercent);
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    onItemClick(event, item) {
        this.legendClick.emit(item.sourceItem);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    legendClass(item) {
        /** @type {?} */
        let ret = ' ';
        if (this.shouldHideChartSelectEffect === false) {
            if (this.selectedItem != null && item.id === this.selectedItem.id) {
                ret += 'legend-selected ';
            }
        }
        if (this.shouldHideChartHoverEffect === false) {
            if (this.hoverItem != null && item.id === this.hoverItem.id) {
                ret += 'legend-hover ';
            }
        }
        return ret;
    }
}
LegendComponent.decorators = [
    { type: Component, args: [{
                selector: 'dl-chart-legend',
                template: "<div>\r\n  <div class=\"legend-items\">\r\n    <div (click)=\"onItemClick($event, item)\" [class]=\"'tooltip legend-item' + legendClass(item)\" *ngFor=\"let item of chartItems\">\r\n      <span id=\"legend-color\" class=\"legend-color-box\" [style.background-color]=\"item.color\"></span>\r\n      <span class=\"legend-text-content \">  \r\n        {{ legendTextValue(item) }}\r\n        <span id=\"dlChartTooltip\" class=\"legend-tooltip tooltiptext \">{{ tooltipTextValue(item) }}</span>\r\n      </span>\r\n    </div>\r\n  </div>\r\n</div>",
                encapsulation: ViewEncapsulation.Emulated,
                styles: [".tooltiptext{position:fixed;overflow:hidden;text-align:center;padding:5px 10px;border-radius:6px;z-index:80;background-color:#555;color:#fff}.legend-items{display:flex;flex-wrap:wrap;margin-left:-10px;margin-top:-10px}.legend-items .legend-item{flex:1 0 auto;margin-left:10px;margin-top:10px;text-align:left;min-width:50px}.legend-items:hover{cursor:default}.legend-color-box{min-height:15px;min-width:15px;display:inline-block;border:1px solid #000}.legend-text-content{min-width:25px;vertical-align:top}.tooltip .legend-tooltip{visibility:hidden;text-align:center;padding:5px 10px;border-radius:6px;position:absolute;z-index:80}.tooltip:hover .legend-tooltip{visibility:visible}.legend-hover{opacity:.8}.legend-selected{font-weight:700}"]
            }] }
];
/** @nocollapse */
LegendComponent.ctorParameters = () => [
    { type: ChartItemService },
    { type: UtilsService },
    { type: ChangeDetectorRef }
];
LegendComponent.propDecorators = {
    legendClick: [{ type: Output }],
    chartidChange: [{ type: Output }],
    chartid: [{ type: Input }],
    hideChartHoverEffect: [{ type: Input }],
    hideChartSelectEffect: [{ type: Input }],
    legendConfiguration: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DlLegendModule {
}
DlLegendModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                ],
                declarations: [
                    LegendComponent
                ],
                providers: [
                    ChartItemService,
                    UtilsService
                ],
                entryComponents: [
                    LegendComponent
                ],
                exports: [
                    LegendComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const ChartOrientation = {
    Bottom: 0,
    Left: 1,
    Right: 2,
    Top: 3,
};
ChartOrientation[ChartOrientation.Bottom] = 'Bottom';
ChartOrientation[ChartOrientation.Left] = 'Left';
ChartOrientation[ChartOrientation.Right] = 'Right';
ChartOrientation[ChartOrientation.Top] = 'Top';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BarChartComponent extends BaseChartComponent {
    /**
     * @param {?} chartItemService
     * @param {?} utilsService
     * @param {?} cd
     */
    constructor(chartItemService, utilsService, cd) {
        super(chartItemService, utilsService, cd);
        this.barWidhtOffset = 5;
        this.currentActiveBar = null;
        this.shouldHideSelectLine = false;
        this.currentScaleLabel = '';
        this.valueSteps = 6;
        this.activeLeftScaleAxis = true;
        this.activeRightScaleAxis = false;
        this.currentOrientation = ChartOrientation.Bottom;
        this.currentBarFullFilled = false;
        this.barGroundLineY = 100;
        this.xAxis = [];
        this.yAxis = [];
        this.bars = [];
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set barFullFilled(val) {
        this.currentBarFullFilled = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set hideSelectLine(val) {
        this.shouldHideSelectLine = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set barOffset(val) {
        this.barWidhtOffset = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set scaleLabel(val) {
        this.currentScaleLabel = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set steps(val) {
        this.valueSteps = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set leftScaleAxis(val) {
        this.activeLeftScaleAxis = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set rightScaleAxis(val) {
        this.activeRightScaleAxis = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set orientation(val) {
        this.currentOrientation = val;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.resetActiveElement();
        this.calculateChart();
    }
    /**
     * @return {?}
     */
    calculateChart() {
        /** @type {?} */
        var items = this.currentValues;
        items.forEach((/**
         * @param {?} val
         * @return {?}
         */
        val => {
            if (!val.value) {
                val.value = 0;
            }
        }));
        /** @type {?} */
        let minValueCalc = Math.min.apply(Math, items.map((/**
         * @param {?} o
         * @return {?}
         */
        function (o) { return o.value; })));
        if (minValueCalc > 0) {
            minValueCalc = 0;
        }
        /** @type {?} */
        let maxValueCalc = Math.max.apply(Math, items.map((/**
         * @param {?} o
         * @return {?}
         */
        function (o) { return o.value; })));
        /** @type {?} */
        let maxValue = (maxValueCalc - minValueCalc);
        /** @type {?} */
        var singleStepValue = (maxValue / this.valueSteps);
        /** @type {?} */
        var singleStepY = (100 / this.valueSteps);
        this.yAxis = this.utilsService.createYAxis(minValueCalc, this.valueSteps, singleStepValue, singleStepY, maxValueCalc);
        this.barGroundLineY = 100;
        if (minValueCalc < 0) {
            this.barGroundLineY = (100 - (100 / maxValue) * (minValueCalc * -1));
        }
        /** @type {?} */
        var singleBarWidht = (((100 - this.barWidhtOffset) - (items.length * this.barWidhtOffset)) / items.length);
        /** @type {?} */
        let bars = [];
        /** @type {?} */
        let index = 1;
        this.currentValues.forEach((/**
         * @param {?} item
         * @return {?}
         */
        item => {
            bars.push({
                val: item,
                position: (this.barWidhtOffset * index) + (singleBarWidht * (index - 1))
            });
            index++;
        }));
        if (maxValue != 0) {
            this.createXAxis(singleBarWidht, bars);
        }
        this.createBars(maxValue, singleBarWidht, bars);
        this.chartItemService.setChartValues(new ServiceItem(this.chartid, this.bars, null, null));
    }
    /**
     * @param {?} singleBarWidht
     * @param {?} items
     * @return {?}
     */
    createXAxis(singleBarWidht, items) {
        this.xAxis = [];
        for (let index = 0; index < items.length; index++) {
            /** @type {?} */
            const element = items[index];
            /** @type {?} */
            var pos = ((element.position + singleBarWidht) - (singleBarWidht / 2));
            this.xAxis.push({
                text: element.val.shortName != null ? element.val.shortName : element.val.name,
                position: pos,
            });
        }
    }
    /**
     * @param {?} maxValue
     * @param {?} singleBarWidht
     * @param {?} items
     * @return {?}
     */
    createBars(maxValue, singleBarWidht, items) {
        /** @type {?} */
        var onePercent = maxValue / 100;
        this.bars = [];
        for (let index = 0; index < items.length; index++) {
            /** @type {?} */
            const element = items[index];
            /** @type {?} */
            var height = element.val.value === 0 ? 0 : (1 * (element.val.value / onePercent));
            /** @type {?} */
            var y = element.val.value === 0 ? 0 : (this.barGroundLineY - (1 * (element.val.value / onePercent)));
            /** @type {?} */
            let isMinusValue = false;
            if (height < 0) {
                isMinusValue = true;
                height = height * -1;
                y = this.barGroundLineY;
            }
            this.bars.push({
                width: singleBarWidht,
                height: height,
                y: y,
                x: element.position,
                sourceItem: element.val,
                calculatedPercent: element.val.value === 0 ? 0 : this.utilsService.roundScale(element.val.value / onePercent),
                color: element.val.color,
                id: this.utilsService.createElementId('chart-bar-', index),
                isMinusValue: isMinusValue
            });
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onClickSegment(event) {
        super.onClickSegment(event);
        if (event === this.currentActiveChartItem) {
            this.currentActiveBar = event;
        }
        else {
            this.currentActiveBar = null;
        }
    }
    /**
     * @return {?}
     */
    resetActiveElement() {
        this.currentActiveBar = null;
        super.resetActive();
    }
    /**
     * @param {?} item
     * @return {?}
     */
    cssClassSegment(item) {
        /** @type {?} */
        let css = '';
        if (this.animationActive) {
            if (this.currentOrientation === ChartOrientation.Bottom || this.currentOrientation === ChartOrientation.Top) {
                css += ' bar-anim-top-bottom';
            }
            else {
                css += ' bar-anim-left-right';
            }
        }
        if (item === this.currentActiveChartItem) {
            css += ' bar-selected';
        }
        if (item.sourceItem) {
            if (item.sourceItem.cssClass != null) {
                css += ' ' + item.sourceItem.cssClass;
            }
        }
        return css;
    }
    /**
     * @return {?}
     */
    get barFillOpacity() {
        if (this.currentBarFullFilled === true) {
            return "1";
        }
        return ".5";
    }
    /**
     * @param {?} defaultValue
     * @param {?} rightValue
     * @param {?} leftValue
     * @param {?} topValue
     * @return {?}
     */
    normOrientation(defaultValue, rightValue, leftValue, topValue) {
        if (this.currentOrientation === ChartOrientation.Bottom) {
            return defaultValue;
        }
        else if (this.currentOrientation === ChartOrientation.Right) {
            return rightValue;
        }
        else if (this.currentOrientation === ChartOrientation.Left) {
            return leftValue;
        }
        else {
            return topValue;
        }
    }
}
BarChartComponent.decorators = [
    { type: Component, args: [{
                selector: 'dl-bar-chart',
                template: "<ng-container>\r\n  <svg class=\"bar-chart-svg\" width=\"100%\" height=\"100%\">\r\n    <svg id=\"bars\" class=\"bars\" x=\"30px\" y=\"30px\" width=\"calc(100% - 60px)\" height=\"calc(100% - 60px)\">\r\n      <rect (mousemove)=\"onHoverSegment($event, bar)\" (mouseleave)=\"onLeaveSegment($event)\" (click)=\"onClickSegment(bar)\"\r\n        [attr.id]=\"bar.id\" [attr.class]=\"'bar ' + cssClassSegment(bar)\" [attr.fill-opacity]=\"barFillOpacity\" [attr.stroke]=\"bar.color\" [attr.fill]=\"bar.color\" [attr.x]=\"normOrientation(bar.x, bar.y, (bar.isMinusValue === false ? (100 - barGroundLineY) : ((100 - barGroundLineY) - bar.height)), bar.x) + '%'\"\r\n        [attr.width]=\"normOrientation(bar.width, bar.height, bar.height, bar.width) + '%'\" [attr.y]=\"normOrientation(bar.y, bar.x, bar.x, (bar.isMinusValue === false ? (100 - barGroundLineY) : ((100 - barGroundLineY) - bar.height))) + '%'\"\r\n        [attr.height]=\"normOrientation(bar.height, bar.width, bar.width, bar.height) + '%'\" *ngFor=\"let bar of bars\">\r\n      </rect>\r\n      <line id=\"dlChartBarSelectLineIndicator\" *ngIf=\"currentActiveBar != null && !shouldHideSelectLine\" [attr.y1]=\"(currentActiveBar.isMinusValue === false ?  normOrientation((barGroundLineY - currentActiveBar.height), 0, 0, ((100 - barGroundLineY) + currentActiveBar.height)) : normOrientation((barGroundLineY + currentActiveBar.height), 0, 0, ((100 - barGroundLineY) - currentActiveBar.height))) + '%'\"\r\n        [attr.x1]=\"(currentActiveBar.isMinusValue === false ?  normOrientation(0, (barGroundLineY - currentActiveBar.height), ((100 - barGroundLineY) + currentActiveBar.height), 0) : normOrientation(0, (barGroundLineY + currentActiveBar.height), ((100 - barGroundLineY) - currentActiveBar.height), 0)) + '%'\"\r\n        [attr.y2]=\"(currentActiveBar.isMinusValue === false ?  normOrientation((barGroundLineY - currentActiveBar.height), 100, 100, ((100 - barGroundLineY) + currentActiveBar.height)) : normOrientation((barGroundLineY + currentActiveBar.height), 100, 100, ((100 - barGroundLineY) - currentActiveBar.height))) + '%'\"\r\n        [attr.x2]=\"(currentActiveBar.isMinusValue === false ?  normOrientation(100, (barGroundLineY - currentActiveBar.height), ((100 - barGroundLineY) + currentActiveBar.height), 100) : normOrientation(100, (barGroundLineY + currentActiveBar.height), ((100 - barGroundLineY) - currentActiveBar.height), 100)) + '%'\"\r\n        class=\"selection-line active-stroke\" />\r\n      <line id=\"dlChartBarNormalizedLine\" [attr.y1]=\"normOrientation(barGroundLineY, 0, 0, (100 - barGroundLineY)) + '%'\"\r\n        [attr.x1]=\"normOrientation(0, barGroundLineY, (100 - barGroundLineY), 0) + '%'\" [attr.y2]=\"normOrientation(barGroundLineY, 100, 100, (100 - barGroundLineY)) + '%'\"\r\n        [attr.x2]=\"normOrientation(100, barGroundLineY, (100 - barGroundLineY), 100) + '%'\" class=\"ground-line\" />\r\n    </svg>\r\n    <svg xaxis [orientation]=\"currentOrientation\" [xAxis]=\"xAxis\"></svg>\r\n    <svg yaxis [activeScaleAxis]=\"activeLeftScaleAxis\" [orientation]=\"currentOrientation\" [yAxis]=\"yAxis\" [scaleLabel]=\"currentScaleLabel\"></svg>\r\n    <svg yaxis [activeScaleAxis]=\"activeRightScaleAxis\" [orientation]=\"currentOrientation\" [yAxis]=\"yAxis\" [isLeft]=\"false\" [scaleLabel]=\"currentScaleLabel\"></svg>\r\n  </svg>\r\n  <div id=\"dlChartTooltip\" *ngIf=\"tooltipShow\" [style.top.px]=\"tooltipTop\" [style.left.px]=\"tooltipLeft\" class=\"tooltiptext\">\r\n    <span>{{ tooltipValue }}</span>\r\n  </div>\r\n</ng-container>",
                encapsulation: ViewEncapsulation.Emulated,
                styles: [".tooltiptext{position:fixed;overflow:hidden;text-align:center;padding:5px 10px;border-radius:6px;z-index:80;background-color:#555;color:#fff}.bar{stroke-width:2px}.bar-selected,.bar:hover{opacity:.8;transition:.5s}.bars line{fill:none;shape-rendering:crispEdges;stroke:#000}.selection-line{fill:#000;stroke-width:1;stroke-dasharray:5}.ground-line{fill:#000}.bar-anim-top-bottom{-webkit-animation:.4s 1ms scaleBar;animation:.4s 1ms reverse scaleBar;-webkit-transform-origin:100% 90% 0;transform-origin:100% 90% 0}.bar-anim-left-right{-webkit-animation:.4s 1ms scaleBarX;animation:.4s 1ms reverse scaleBarX;-webkit-transform-origin:100% 90% 0;transform-origin:100% 90% 0}@-webkit-keyframes scaleBar{100%{-webkit-transform:scaleY(0);transform:scaleY(0)}}@keyframes scaleBar{100%{-webkit-transform:scaleY(0);transform:scaleY(0)}}@-webkit-keyframes scaleBarX{100%{-webkit-transform:scaleX(0);transform:scaleX(0)}}@keyframes scaleBarX{100%{-webkit-transform:scaleX(0);transform:scaleX(0)}}"]
            }] }
];
/** @nocollapse */
BarChartComponent.ctorParameters = () => [
    { type: ChartItemService },
    { type: UtilsService },
    { type: ChangeDetectorRef }
];
BarChartComponent.propDecorators = {
    barFullFilled: [{ type: Input }],
    hideSelectLine: [{ type: Input }],
    barOffset: [{ type: Input }],
    scaleLabel: [{ type: Input }],
    steps: [{ type: Input }],
    leftScaleAxis: [{ type: Input }],
    rightScaleAxis: [{ type: Input }],
    orientation: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class YAxisComponent {
    constructor() {
        this.currentOrientation = ChartOrientation.Bottom;
        this.currentActiveScaleAxis = true;
        this.currentYAxis = [];
        this.currentScaleLabel = '';
        this.currentIsLeft = true;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set orientation(val) {
        this.currentOrientation = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set yAxis(val) {
        this.currentYAxis = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set activeScaleAxis(val) {
        this.currentActiveScaleAxis = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set isLeft(val) {
        this.currentIsLeft = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set scaleLabel(val) {
        this.currentScaleLabel = val;
    }
    /**
     * @param {?} defaultValue
     * @param {?} rightValue
     * @param {?} leftValue
     * @param {?} topValue
     * @param {?} defaultValueRight
     * @param {?} rightValueRight
     * @param {?} leftValueRight
     * @param {?} topValueRight
     * @return {?}
     */
    normOrientation(defaultValue, rightValue, leftValue, topValue, defaultValueRight, rightValueRight, leftValueRight, topValueRight) {
        if (this.currentOrientation === ChartOrientation.Bottom) {
            if (this.currentIsLeft) {
                return defaultValue;
            }
            else {
                return defaultValueRight;
            }
        }
        else if (this.currentOrientation === ChartOrientation.Right) {
            if (this.currentIsLeft) {
                return rightValue;
            }
            else {
                return rightValueRight;
            }
        }
        else if (this.currentOrientation === ChartOrientation.Left) {
            if (this.currentIsLeft) {
                return leftValue;
            }
            else {
                return leftValueRight;
            }
        }
        else {
            if (this.currentIsLeft) {
                return topValue;
            }
            else {
                return topValueRight;
            }
        }
    }
}
YAxisComponent.decorators = [
    { type: Component, args: [{
                selector: '[yaxis]',
                template: "<svg id=\"yaxis\" \r\n[attr.x]=\"normOrientation('0%', '30px', '30px', 'calc(100% - 30px)', 'calc(100% - 30px)', '30px', '30px', '0%')\" \r\n[attr.y]=\"normOrientation('30px', 'calc(100% - 30px)', '0px', '30px', '30px', '0px', 'calc(100% - 30px)', '30px')\" \r\n[attr.width]=\"normOrientation('30px', 'calc(100% - 60px)', 'calc(100% - 60px)', '30px', '30px', 'calc(100% - 60px)', 'calc(100% - 60px)', '30px')\" \r\n[attr.height]=\"normOrientation('calc(100% - 60px)', '30px', '30px', 'calc(100% - 60px)', 'calc(100% - 60px)', '30px', '30px', 'calc(100% - 60px)')\" style=\"overflow: visible;\">\r\n  <g class=\"y axis\" *ngIf=\"currentActiveScaleAxis\">\r\n    <g class=\"tick\" *ngFor=\"let axis of currentYAxis\">\r\n      <line [attr.x1]=\"normOrientation('calc(100% - 6px)', axis.position + '%', (100 - axis.position) + '%', 0, 0, axis.position + '%', (100 - axis.position) + '%', 'calc(100% - 6px)')\" \r\n      [attr.x2]=\"normOrientation('100%', axis.position + '%', (100 - axis.position) + '%', 6, 6, axis.position + '%', (100 - axis.position) + '%', '100%')\" \r\n      [attr.y1]=\"normOrientation(axis.position + '%', '0', '30px', (100 - axis.position) + '%', axis.position + '%', '30px', 0, (100 - axis.position) + '%')\" \r\n      [attr.y2]=\"normOrientation(axis.position + '%', '6', '24px', (100 - axis.position) + '%', axis.position + '%', '24px', 6, (100 - axis.position) + '%')\"></line>\r\n      <text dy=\".32em\" \r\n      [attr.x]=\"normOrientation('75%', axis.position + '%', (100 - axis.position) + '%', '8', '8', axis.position + '%', (100 - axis.position) + '%', '75%')\" \r\n      [attr.y]=\"normOrientation(axis.position + '%', '12px', '18px', (100 - axis.position) + '%', axis.position + '%', '18px', '12px', (100 - axis.position) + '%')\" \r\n      [style.text-anchor]=\"normOrientation('end', 'middle', 'middle', 'start', 'start', 'middle', 'middle', 'end')\">{{ axis.text }}</text>\r\n    </g>\r\n    <line\r\n    [attr.x1]=\"normOrientation('100%', '0', '0', 0, 0, '0', 0, '100%')\" \r\n    [attr.x2]=\"normOrientation('100%', '100%', '100%', 0, 0, '100%', '100%', '100%')\" \r\n    [attr.y1]=\"normOrientation('0%', '0', '30px', '0%', '0%', '30px', 0, '0%')\" \r\n    [attr.y2]=\"normOrientation('100%', '0', '30px', '100%', '100%', '30px', 0, '100%')\" \r\n    class=\"x-stroke\" />\r\n  </g>\r\n</svg>\r\n<svg *ngIf=\"currentActiveScaleAxis\" \r\n[attr.x]=\"normOrientation('31px', '30px', 'calc(100% - 30px)', 'calc(100% - 43px)', 'calc(100% - 43px)', '30px', 'calc(100% - 31px)', '31px')\" \r\n[attr.y]=\"normOrientation('30px', 'calc(100% - 43px)', '31px', 'calc(100% - 31px)', '30px', '31px', 'calc(100% - 43px)', 'calc(100% - 31px)')\">\r\n  <text dy=\".71em\" \r\n  [attr.x]=\"'0'\" \r\n  [attr.y]=\"'0'\" \r\n  [ngStyle]=\"{'transform': normOrientation('rotate(-90deg)', 'rotate(0deg)', 'rotate(0deg)', 'rotate(-90deg)', 'rotate(-90deg)', 'rotate(0deg)', 'rotate(0deg)', 'rotate(-90deg)'), 'text-anchor': normOrientation('end', 'start', 'end', 'start', 'end', 'start', 'end', 'start')}\">{{ currentScaleLabel }}</text>\r\n</svg>",
                styles: [".axis{font:10px sans-serif}.axis line,.axis path{fill:none;stroke:#000}.axis text{fill:#000}"]
            }] }
];
YAxisComponent.propDecorators = {
    orientation: [{ type: Input }],
    yAxis: [{ type: Input }],
    activeScaleAxis: [{ type: Input }],
    isLeft: [{ type: Input }],
    scaleLabel: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class XAxisComponent {
    constructor() {
        this.currentOrientation = ChartOrientation.Bottom;
        this.currentXAxis = [];
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set orientation(val) {
        this.currentOrientation = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set xAxis(val) {
        this.currentXAxis = val;
    }
    /**
     * @param {?} defaultValue
     * @param {?} rightValue
     * @param {?} leftValue
     * @param {?} topValue
     * @return {?}
     */
    normOrientation(defaultValue, rightValue, leftValue, topValue) {
        if (this.currentOrientation === ChartOrientation.Bottom) {
            return defaultValue;
        }
        else if (this.currentOrientation === ChartOrientation.Right) {
            return rightValue;
        }
        else if (this.currentOrientation === ChartOrientation.Left) {
            return leftValue;
        }
        else {
            return topValue;
        }
    }
}
XAxisComponent.decorators = [
    { type: Component, args: [{
                selector: '[xaxis]',
                template: "<svg id=\"xaxis\" [attr.x]=\"normOrientation('30px', 'calc(100% - 30px)', '0%', '30px')\"\r\n  [attr.y]=\"normOrientation('calc(100% - 30px)', '30px', '30px', '0px')\"\r\n  [attr.width]=\"normOrientation('calc(100% - 60px)', '30px', '30px', 'calc(100% - 60px)')\"\r\n  [attr.height]=\"normOrientation( '30px', 'calc(100% - 60px)', 'calc(100% - 60px)', '30px')\" style=\"overflow: visible;\">\r\n  <g class=\"x axis xaxis-text\">\r\n    <g class=\"tick\" *ngFor=\"let axis of currentXAxis\">\r\n      <line [attr.x1]=\"normOrientation(axis.position + '%', '0', 'calc(100% - 6px)', (100 - axis.position) + '%')\"\r\n        [attr.x2]=\"normOrientation(axis.position + '%', 4, '100%', (100 - axis.position) + '%')\"\r\n        [attr.y1]=\"normOrientation(0, axis.position + '%', (100 - axis.position) + '%', '30px')\"\r\n        [attr.y2]=\"normOrientation(4, axis.position + '%', (100 - axis.position) + '%', '24px')\"></line>\r\n      <text [attr.dy]=\"normOrientation('.71em', '.31em', '.32em', '.32em')\"\r\n        [attr.x]=\"normOrientation(axis.position + '%', '6', '75%', (100 - axis.position) + '%')\"\r\n        [attr.y]=\"normOrientation('6', axis.position + '%', axis.position + '%', '18px')\"\r\n        [ngStyle]=\"{'text-anchor': normOrientation('middle', 'start', 'end', 'middle')}\">{{\r\n            axis.text }}</text>\r\n    </g>\r\n    <line id=\"dlChartBarXLine\" [attr.x1]=\"normOrientation(0, 0, '100%', '0')\"\r\n      [attr.y1]=\"normOrientation(0, 0, '0%', '30px')\" [attr.x2]=\"normOrientation('100%', 0, '100%', '100%')\"\r\n      [attr.y2]=\"normOrientation(0, '100%', '100%', '30px')\" class=\"x-stroke\" />\r\n  </g>\r\n</svg>",
                styles: [".axis{font:10px sans-serif}.axis line,.axis path{fill:none;stroke:#000}.axis text{fill:#000}"]
            }] }
];
XAxisComponent.propDecorators = {
    orientation: [{ type: Input }],
    xAxis: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AxisComponentsModule {
}
AxisComponentsModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [
                    YAxisComponent,
                    XAxisComponent
                ],
                providers: [],
                entryComponents: [],
                exports: [
                    YAxisComponent,
                    XAxisComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DlBarChartModule {
}
DlBarChartModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    AxisComponentsModule,
                    CommonModule
                ],
                declarations: [
                    BarChartComponent
                ],
                providers: [
                    ChartItemService,
                    UtilsService
                ],
                entryComponents: [
                    BarChartComponent
                ],
                exports: [
                    BarChartComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LineChartComponent extends BaseChartComponent {
    /**
     * @param {?} chartItemService
     * @param {?} utilsService
     * @param {?} cd
     */
    constructor(chartItemService, utilsService, cd) {
        super(chartItemService, utilsService, cd);
        this.currentScaleLabel = '';
        this.valueSteps = 6;
        this.activeLeftScaleAxis = true;
        this.activeRightScaleAxis = false;
        this.activeHideRaster = false;
        this.activeHideLines = false;
        this.activeHideLinesFiller = false;
        this.activeHidePoints = false;
        this.activeHideSelectionLines = false;
        this.xAxis = [];
        this.yAxis = [];
        this.axisPoint = [];
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set scaleLabel(val) {
        this.currentScaleLabel = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set steps(val) {
        this.valueSteps = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set leftScaleAxis(val) {
        this.activeLeftScaleAxis = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set rightScaleAxis(val) {
        this.activeRightScaleAxis = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set hideRaster(val) {
        this.activeHideRaster = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set hideLines(val) {
        this.activeHideLines = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set hideLinesFiller(val) {
        this.activeHideLinesFiller = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set hidePoints(val) {
        this.activeHidePoints = val;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set hideSelectionLines(val) {
        this.activeHideSelectionLines = val;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.steps == undefined && changes.values != undefined && ((/** @type {?} */ (changes.values.currentValue))) != undefined && ((/** @type {?} */ (changes.values.previousValue))) != undefined) {
            if (!this.compare(changes.values.currentValue, changes.values.previousValue)) {
                this.resetActiveElement();
                this.calculateChart();
            }
        }
        else {
            this.resetActiveElement();
            this.calculateChart();
        }
    }
    /**
     * @param {?} arr1
     * @param {?} arr2
     * @return {?}
     */
    compare(arr1, arr2) {
        if (!arr1 || !arr2)
            return;
        /** @type {?} */
        let result = false;
        arr1.forEach((/**
         * @param {?} e1
         * @param {?} i
         * @return {?}
         */
        (e1, i) => arr2.forEach((/**
         * @param {?} e2
         * @return {?}
         */
        e2 => {
            if (e1.length > 1 && e2.length) {
                result = this.compare(e1, e2);
            }
            else if (e1 !== e2) {
                result = false;
            }
            else {
                result = true;
            }
        }))));
        return result;
    }
    /**
     * @return {?}
     */
    calculateChart() {
        /** @type {?} */
        var items = this.currentValues;
        /** @type {?} */
        let uniqueXPoints = [];
        /** @type {?} */
        let uniqueYPoints = [];
        items.forEach((/**
         * @param {?} val
         * @return {?}
         */
        val => {
            val.points.forEach((/**
             * @param {?} point
             * @return {?}
             */
            point => {
                if (!uniqueXPoints.some((/**
                 * @param {?} e
                 * @return {?}
                 */
                e => e === point.xValue))) {
                    uniqueXPoints.push(point.xValue);
                }
                if (!uniqueYPoints.some((/**
                 * @param {?} e
                 * @return {?}
                 */
                e => e === point.yValue))) {
                    uniqueYPoints.push(point.yValue);
                }
            }));
        }));
        /** @type {?} */
        let xMinValue = Math.min.apply(Math, uniqueXPoints.map((/**
         * @param {?} o
         * @return {?}
         */
        function (o) { return o; })));
        /** @type {?} */
        let xMaxValue = Math.max.apply(Math, uniqueXPoints.map((/**
         * @param {?} o
         * @return {?}
         */
        function (o) { return o; })));
        /** @type {?} */
        let onePercentX = (xMaxValue - xMinValue) / 100;
        /** @type {?} */
        let yMinValue = Math.min.apply(Math, uniqueYPoints.map((/**
         * @param {?} o
         * @return {?}
         */
        function (o) { return o; })));
        /** @type {?} */
        let yMaxValue = Math.max.apply(Math, uniqueYPoints.map((/**
         * @param {?} o
         * @return {?}
         */
        function (o) { return o; })));
        /** @type {?} */
        var useMaxValueY = (yMaxValue - yMinValue);
        /** @type {?} */
        var singleStepY = (100 / this.valueSteps);
        /** @type {?} */
        var singleStepYValue = (useMaxValueY / this.valueSteps);
        this.yAxis = this.utilsService.createYAxis(yMinValue, this.valueSteps, singleStepYValue, singleStepY, yMaxValue);
        // sort the uniqueXPoints
        uniqueXPoints = uniqueXPoints.sort((/**
         * @param {?} a
         * @param {?} b
         * @return {?}
         */
        (a, b) => {
            if (a > b) {
                return 0;
            }
            return 1;
        }));
        /** @type {?} */
        var xA = [];
        /** @type {?} */
        let index = 0;
        for (index = 0; index < (uniqueXPoints.length); index++) {
            xA.push(uniqueXPoints[index].toString());
        }
        this.xAxis = this.createAxis(xA, 100);
        this.axisPoint = [];
        /** @type {?} */
        var oneValueYPercent = useMaxValueY / 100;
        for (index = 0; index < items.length; index++) {
            /** @type {?} */
            const element = items[index];
            /** @type {?} */
            let pointAxis = [];
            /** @type {?} */
            let indexPoints = 0;
            /** @type {?} */
            let subItemId = this.utilsService.createElementId('chart-line-point-', index);
            element.points.forEach((/**
             * @param {?} point
             * @return {?}
             */
            point => {
                /** @type {?} */
                let x = ((point.xValue - xMinValue) / onePercentX);
                /** @type {?} */
                var percentValueY = this.utilsService.roundScale((point.yValue - yMinValue) / oneValueYPercent);
                /** @type {?} */
                var displayValueY = this.utilsService.roundScale(percentValueY * 1);
                /** @type {?} */
                let y = 100 - displayValueY;
                if (point.name === '' || point.name === null || point.name === undefined) {
                    point.name = element.name;
                }
                pointAxis.push({
                    x: x,
                    y: y,
                    size: 5,
                    sourceItem: point,
                    calculatedPercent: this.utilsService.roundScale((point.xValue - xMinValue) / onePercentX),
                    color: point.color === null ? element.color : point.color,
                    id: this.utilsService.createElementId(subItemId + '-', indexPoints),
                });
                indexPoints++;
            }));
            this.axisPoint.push({
                points: pointAxis,
                sourceItem: element,
                calculatedPercent: null,
                color: element.color,
                id: this.utilsService.createElementId('chart-line-axis-', index),
            });
        }
        this.chartItemService.setChartValues(new ServiceItem(this.chartid, this.axisPoint, null, null));
    }
    /**
     * @param {?} points
     * @return {?}
     */
    getPathFromAxisLine(points) {
        /** @type {?} */
        let path = "";
        points.forEach((/**
         * @param {?} line
         * @return {?}
         */
        line => {
            if (path != "") {
                path += " ";
            }
            path += line.x + "," + line.y;
        }));
        return path;
    }
    /**
     * @param {?} items
     * @param {?} length
     * @return {?}
     */
    createAxis(items, length) {
        /** @type {?} */
        let axis = [];
        /** @type {?} */
        var step = (length / (items.length - 1));
        for (let index = 0; index < (items.length); index++) {
            /** @type {?} */
            const element = items[index];
            axis.push({
                text: element,
                position: (100 - (length - (step * index))),
            });
        }
        return axis;
    }
    /**
     * @return {?}
     */
    resetActiveElement() {
        super.resetActive();
    }
    /**
     * @param {?} event
     * @param {?} aPoint
     * @return {?}
     */
    onClickSegmentPoint(event, aPoint) {
        event.id = aPoint.id;
        super.onClickSegment(event);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    cssClassSegmentLine(item) {
        /** @type {?} */
        let css = '';
        if (this.animationActive) {
            css += ' line-anim';
        }
        if (item === this.currentActiveChartItem) {
            css += ' ';
        }
        if (item.sourceItem.cssClass != null) {
            css += ' ' + item.sourceItem.cssClass;
        }
        return css;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    cssClassSegmentPoint(item) {
        /** @type {?} */
        let css = '';
        if (this.animationActive) {
            css += ' point-anim';
        }
        if (item === this.currentActiveChartItem) {
            css += ' point-selected';
        }
        if (item.sourceItem.cssClass != null) {
            css += ' ' + item.sourceItem.cssClass;
        }
        return css;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    pointSize(item) {
        if (item === this.currentActiveChartItem) {
            return item.size + 2;
        }
        return item.size;
    }
    /**
     * @return {?}
     */
    get currentActivePoint() {
        if (this.currentActiveChartItem != null) {
            /** @type {?} */
            let point = (/** @type {?} */ (this.currentActiveChartItem));
            if (point != undefined && point.x != undefined) {
                return point;
            }
        }
        return null;
    }
    /**
     * @return {?}
     */
    get selectionXEndPoint() {
        if (this.currentActiveChartItem != null) {
            /** @type {?} */
            let point = (/** @type {?} */ (this.currentActiveChartItem));
            if (point.x > 50) {
                if (this.activeRightScaleAxis === true) {
                    return 100;
                }
                else {
                    return 0;
                }
            }
            else {
                if (this.activeLeftScaleAxis === true) {
                    return 0;
                }
                else {
                    return 100;
                }
            }
        }
    }
}
LineChartComponent.decorators = [
    { type: Component, args: [{
                selector: 'dl-line-chart',
                template: "<ng-container>\r\n  <svg class=\"line-chart-svg\" width=\"100%\" height=\"100%\">\r\n    <svg xaxis [orientation]=\"0\" [xAxis]=\"xAxis\"></svg>\r\n    <svg yaxis [activeScaleAxis]=\"activeLeftScaleAxis\" [orientation]=\"0\" [yAxis]=\"yAxis\" [scaleLabel]=\"currentScaleLabel\"></svg>\r\n    <svg yaxis [activeScaleAxis]=\"activeRightScaleAxis\" [orientation]=\"0\" [yAxis]=\"yAxis\" [isLeft]=\"false\" [scaleLabel]=\"currentScaleLabel\"></svg>\r\n    <svg width=\"calc(100% - 60px)\" height=\"calc(100% - 60px)\" x=\"30px\" y=\"30px\" style=\"overflow: visible;\">\r\n      <g *ngIf=\"!activeHideRaster\" class=\"raster\" id=\"background-raster\">\r\n        <line class=\"background-raster\" [attr.x1]=\"axis.position + '%'\" [attr.x2]=\"axis.position + '%'\" y1=\"0\" y2=\"100%\"\r\n          *ngFor=\"let axis of xAxis\"></line>\r\n        <line class=\"background-raster\" [attr.y1]=\"axis.position + '%'\" [attr.y2]=\"axis.position + '%'\" x1=\"0\" x2=\"100%\"\r\n          *ngFor=\"let axis of yAxis\"></line>\r\n      </g>\r\n      <svg *ngIf=\"!activeHideLines\" width=\"100%\" height=\"100%\" viewBox=\"0 0 100 100\" preserveAspectRatio=\"none\" style=\"overflow:visible\">\r\n        <g *ngFor=\"let axis of axisPoint\">\r\n          <polyline class=\"line cssClassSegmentLine(axis)\" fill=\"none\" [attr.stroke]=\"axis.color\" stroke-width=\"1\" [attr.points]=\"getPathFromAxisLine(axis.points)\" />\r\n          <polygon *ngIf=\"!activeHideLinesFiller\" class=\"line-filler\" [attr.fill]=\"axis.color\" stroke=\"none\"\r\n            [attr.points]=\"getPathFromAxisLine(axis.points) + ' 100,100 0,100'\" />\r\n        </g>\r\n      </svg>\r\n      <g *ngIf=\"!activeHideSelectionLines && currentActivePoint != null\" id=\"selection\">\r\n        <line class=\"point-selected x-point-selected\" x2=\"0\" [attr.y2]=\"currentActivePoint.y + '%'\" [attr.x2]=\"currentActivePoint.x + '%'\"\r\n          [attr.y1]=\"'100%'\" [attr.x1]=\"currentActivePoint.x + '%'\"></line>\r\n        <line class=\"point-selected y-point-selected\" [attr.x1]=\"selectionXEndPoint + '%'\" [attr.x2]=\"currentActivePoint.x+ '%'\"\r\n          [attr.y2]=\"currentActivePoint.y+ '%'\" [attr.y1]=\"currentActivePoint.y+ '%'\"></line>\r\n      </g>\r\n      <g [attr.class]=\"'line-axis line-axis-anim'\" *ngFor=\"let axis of axisPoint\">\r\n        <g *ngIf=\"!activeHidePoints\" id=\"selection-point\">\r\n          <circle *ngFor=\"let point of axis.points\" (mousemove)=\"onHoverSegment($event, point)\" (mouseleave)=\"onLeaveSegment($event)\"\r\n            (click)=\"onClickSegmentPoint(point, axis)\" [attr.id]=\"point.id\" [attr.class]=\"'point ' + cssClassSegmentPoint(point)\"\r\n            [attr.fill]=\"point.color\" [attr.cx]=\"point.x + '%'\" [attr.cy]=\"point.y + '%'\" [attr.r]=\"pointSize(point)\" />\r\n        </g>\r\n      </g>\r\n    </svg>\r\n  </svg>\r\n  <div id=\"dlChartTooltip\" *ngIf=\"tooltipShow\" [style.top.px]=\"tooltipTop\" [style.left.px]=\"tooltipLeft\" class=\"tooltiptext\">\r\n    <span>{{ tooltipValue }}</span>\r\n  </div>\r\n</ng-container>",
                encapsulation: ViewEncapsulation.Emulated,
                styles: [".tooltiptext{position:fixed;overflow:hidden;text-align:center;padding:5px 10px;border-radius:6px;z-index:80;background-color:#555;color:#fff}.raster line{fill:none;stroke:#000}.background-raster{opacity:.1}.point-anim{transition:.5s;-webkit-animation:.4s showPoint;animation:.4s reverse showPoint}.line{opacity:.8}.line-filler{opacity:.3}.line-anim{transition:.5s;stroke-dasharray:1000;stroke-dashoffset:1000;-webkit-animation:.5s linear forwards dash;animation:.5s linear forwards dash}.point-selected{transition:.5s;stroke-width:3px;stroke:#000}@-webkit-keyframes showPoint{100%{opacity:0}}@keyframes showPoint{100%{opacity:0}}@-webkit-keyframes dash{from{stroke-dashoffset:1000}to{stroke-dashoffset:0}}@keyframes dash{from{stroke-dashoffset:1000}to{stroke-dashoffset:0}}"]
            }] }
];
/** @nocollapse */
LineChartComponent.ctorParameters = () => [
    { type: ChartItemService },
    { type: UtilsService },
    { type: ChangeDetectorRef }
];
LineChartComponent.propDecorators = {
    scaleLabel: [{ type: Input }],
    steps: [{ type: Input }],
    leftScaleAxis: [{ type: Input }],
    rightScaleAxis: [{ type: Input }],
    hideRaster: [{ type: Input }],
    hideLines: [{ type: Input }],
    hideLinesFiller: [{ type: Input }],
    hidePoints: [{ type: Input }],
    hideSelectionLines: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DlLineChartModule {
}
DlLineChartModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    AxisComponentsModule,
                    CommonModule
                ],
                declarations: [
                    LineChartComponent
                ],
                providers: [
                    ChartItemService,
                    UtilsService
                ],
                entryComponents: [
                    LineChartComponent
                ],
                exports: [
                    LineChartComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DlChartModule {
}
DlChartModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    DlPieChartModule,
                    DlLegendModule,
                    DlBarChartModule,
                    DlLineChartModule
                ],
                declarations: [],
                providers: [],
                exports: [
                    DlPieChartModule,
                    DlLegendModule,
                    DlBarChartModule,
                    DlLineChartModule
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Value {
    /**
     * @param {?} name
     * @param {?} value
     * @param {?} color
     * @param {?=} cssClass
     * @param {?=} tooltipConfig
     * @param {?=} data
     * @param {?=} shortName
     */
    constructor(name, value, color, cssClass = null, tooltipConfig = null, data = null, shortName = null) {
        this.shortName = null;
        this.cssClass = null;
        this.tooltipConfig = null;
        this.data = null;
        this.name = name;
        this.value = value;
        this.color = color;
        this.cssClass = cssClass;
        this.tooltipConfig = tooltipConfig;
        this.data = data;
        this.shortName = shortName;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Point {
    /**
     * @param {?} xValue
     * @param {?} yValue
     * @param {?=} name
     * @param {?=} color
     * @param {?=} cssClass
     * @param {?=} tooltipConfig
     * @param {?=} data
     */
    constructor(xValue, yValue, name = null, color = null, cssClass = null, tooltipConfig = null, data = null) {
        this.name = null;
        this.color = null;
        this.cssClass = null;
        this.tooltipConfig = null;
        this.data = null;
        this.xValue = xValue;
        this.yValue = yValue;
        this.name = name;
        this.color = color;
        this.cssClass = cssClass;
        this.tooltipConfig = tooltipConfig;
        this.data = data;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class Line {
    /**
     * @param {?} name
     * @param {?} points
     * @param {?} color
     * @param {?=} cssClass
     * @param {?=} tooltipConfig
     * @param {?=} data
     */
    constructor(name, points, color, cssClass = null, tooltipConfig = null, data = null) {
        this.cssClass = null;
        this.tooltipConfig = null;
        this.data = null;
        this.name = name;
        this.points = points;
        this.color = color;
        this.cssClass = cssClass;
        this.tooltipConfig = tooltipConfig;
        this.data = data;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ChartItemService, ChartOrientation, DlBarChartModule, DlChartModule, DlLegendModule, DlLineChartModule, DlPieChartModule, LegendConfiguration, Line, Point, ServiceItem, TooltipConfiguration, Value, PieChartComponent as ɵa, BaseChartComponent as ɵb, UtilsService as ɵc, LegendComponent as ɵd, AxisComponentsModule as ɵe, YAxisComponent as ɵf, XAxisComponent as ɵg, BarChartComponent as ɵh, LineChartComponent as ɵi };
//# sourceMappingURL=dl-chart.js.map
